#include "stdafx.h"
#include "CelestialBody.h"
#include "glext.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCelestialBody::CCelestialBody()
:pParent(NULL), planetName (""),  orbitDistance(0.0f), orbitSpeed(0.0f),
Radius(1.0f), rotationalSpeed(0.0), lastUpdate(0),
Shininess(1.0), Slices(30), Stacks(30), 
currentRotation(0.0f), currentOrbit(0.0f), sunLight(GL_LIGHT1), 
isSun(false), tiltPlanet(0.0f)
{
	pAmbient[0]=0.2;
	pAmbient[1]=0.2;
	pAmbient[2]=0.2;
	pAmbient[3] = 1.0;

	pDiffuse[0]=0.8;
	pDiffuse[1]=0.8;
	pDiffuse[2]=0.8;
	pDiffuse[3] = 1.0;

	pSpecular[0]=0.0;
	pSpecular[1]=0.0;
	pSpecular[2]=0.0;
	pSpecular[3] = 1.0;

	pEmission[0]=0.0;
	pEmission[1]=0.0;
	pEmission[2]=0.0;
	pEmission[3] = 1.0;

	planeTiltAxis[0]=1.0;
	planeTiltAxis[1]=0.0;
	planeTiltAxis[2]=0.0;
}

CCelestialBody::~CCelestialBody()
{
	CCelestialBodyList::iterator itr;

	for (itr = listChildren.begin(); itr != listChildren.end(); itr++)
	{
		CCelestialBody* pChild = *itr;
		delete pChild;
	}
}

// Updates the world position of the planet, then repositions the children
void CCelestialBody::Update()
{
	glPushMatrix();
	glRotatef(tiltPlanet, planeTiltAxis[0], planeTiltAxis[1], planeTiltAxis[2]);
	glRotatef(currentOrbit, 0.0, 1.0, 0.0);
	glTranslatef(orbitDistance, 0.0, 0.0);
	glPushMatrix();
	if (isSun)
	{
		float pLightDir[] = {0.0, 0.0, 1.0, 0.0};
		float pLightDiffuse[] = {1.0, 1.0, 0.0, 1.0};
		float pLightSpecular[] = {0.3, 0.0, 1.0, 1.0};
		glEnable(sunLight);
		glLightfv(sunLight, GL_POSITION, pLightDir);
		glLightfv(sunLight, GL_DIFFUSE, pLightDiffuse);
		glLightfv(sunLight, GL_SPECULAR, pLightSpecular);
	}

	glRotatef(currentRotation, 0.0, 1.0, 0.0);
	DrawBody();
	if (isSun)
	{
		glDisable(sunLight);
	}
	glPopMatrix();

	CCelestialBodyList::iterator itr;

	for (itr = listChildren.begin(); itr != listChildren.end(); itr++)
	{
		CCelestialBody* pChild = *itr;

		pChild->Update();
	}
	glPopMatrix();

	// Update the rotations
	if (lastUpdate != 0)
	{
		float fSecs = static_cast<float>(GetTime() - lastUpdate)/1000.0;

		currentOrbit += orbitSpeed * fSecs;
		currentRotation += rotationalSpeed * fSecs;

		if (currentOrbit > 360.0)
			currentOrbit -= 360.0;

		if (currentOrbit < -360.0)
			currentOrbit +=360.0;

		if (currentRotation > 360.0)
			currentRotation -= 360.0;

		if (currentRotation < -360.0)
			currentRotation +=360.0;

	}
	lastUpdate = GetTime();
}

// Creates a satelite object
CCelestialBody* CCelestialBody::CreateChild()
{
	CCelestialBody* pChild = new CCelestialBody;
	pChild->pParent = this;
	listChildren.push_back(pChild);
	return pChild;
}

void CCelestialBody::SetMaterialProperty(GLenum eProp, float *pVal)
{
	switch(eProp)
	{
	case GL_AMBIENT_AND_DIFFUSE:
		pAmbient[0] = pDiffuse[0] = pVal[0];
		pAmbient[1] = pDiffuse[1] = pVal[1];
		pAmbient[2] = pDiffuse[2] = pVal[2];
		pAmbient[3] = pDiffuse[3] = pVal[3];
		break;
	case GL_AMBIENT:
		pAmbient[0] = pVal[0];
		pAmbient[1] = pVal[1];
		pAmbient[2] = pVal[2];
		pAmbient[3] = pVal[3];
		break;
	case GL_DIFFUSE:
		pDiffuse[0] = pVal[0];
		pDiffuse[1] = pVal[1];
		pDiffuse[2] = pVal[2];
		pDiffuse[3] = pVal[3];
		break;
	case GL_SPECULAR:
		pSpecular[0] = pVal[0];
		pSpecular[1] = pVal[1];
		pSpecular[2] = pVal[2];
		pSpecular[3] = pVal[3];
		break;		
	case GL_EMISSION:
		pEmission[0] = pVal[0];
		pEmission[1] = pVal[1];
		pEmission[2] = pVal[2];
		pEmission[3] = pVal[3];
		break;
	case GL_SHININESS:
		Shininess = *pVal;
		break;
	}
}

void CCelestialBody::GetMaterialProp(GLenum eProp, float *pVal) const
{
	switch(eProp)
	{
	case GL_AMBIENT_AND_DIFFUSE:
		pVal[0] = (pAmbient[0] + pDiffuse[0]) / 2.0;
		pVal[1] = (pAmbient[1] + pDiffuse[1]) / 2.0;
		pVal[2] = (pAmbient[2] + pDiffuse[2]) / 2.0;
		pVal[3] = (pAmbient[3] + pDiffuse[3]) / 2.0;
		break;
	case GL_AMBIENT:
		pVal[0] = pAmbient[0];
		pVal[1] = pAmbient[1];
		pVal[2] = pAmbient[2];
		pVal[3] = pAmbient[3];
		break;
	case GL_DIFFUSE:
		pVal[0] = pDiffuse[0];
		pVal[1] = pDiffuse[1];
		pVal[2] = pDiffuse[2];
		pVal[3] = pDiffuse[3];
		break;
	case GL_SPECULAR:
		pVal[0] = pSpecular[0];
		pVal[1] = pSpecular[1];
		pVal[2] = pSpecular[2];
		pVal[3] = pSpecular[3];
		break;		
	case GL_EMISSION:
		pVal[0] = pEmission[0];
		pVal[1] = pEmission[1];
		pVal[2] = pEmission[2];
		pVal[3] = pEmission[3];
		break;
	case GL_SHININESS:
		pVal[0] = Shininess;
		break;
	}
}

void CCelestialBody::DrawBody()
{

	glMaterialfv(GL_FRONT, GL_AMBIENT, pAmbient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, pDiffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, pSpecular);
	glMaterialfv(GL_FRONT, GL_EMISSION, pEmission);
	glMaterialf(GL_FRONT, GL_SHININESS, Shininess);

	GLUquadricObj* pSphere = gluNewQuadric();
	gluQuadricNormals(pSphere, GLU_SMOOTH);
	gluSphere(pSphere, Radius, Slices, Stacks);
	gluDeleteQuadric(pSphere);
}

int CCelestialBody::GetTime()
{
	timeb t;
	ftime(&t);

	return 1000 * t.time + t.millitm;
}
