////////////////////////////////////////////////////////////////////

 //CelestialBody.h: interface for the CCelestialBody class.

 //Stores the information for creating a planet or sun and any satellites
 //that orbit it

////////////////////////////////////////////////////////////////////

#if !defined(AFX_PLANET_H__820496D2_8A07_45EA_8E0B_2416C1485B26__INCLUDED_)
#define AFX_PLANET_H__820496D2_8A07_45EA_8E0B_2416C1485B26__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <list>

class CCelestialBody;

typedef std::list<CCelestialBody*>	CCelestialBodyList;

class CCelestialBody  
{
public:
	
	void GetMaterialProp(GLenum eProp, float* pVal) const;
	void SetMaterialProperty(GLenum eProp, float* pVal);

	CCelestialBody* CreateChild();
	void SetPlanetName (string pName) {planetName = pName;}
	void SetRotationalSpeed(float fRotate) {rotationalSpeed = fRotate;}
	void SetOrbitSpeed(float fOrbit) {orbitSpeed = fOrbit;}
	void SetOrbitDistance(float fDistance) {orbitDistance = fDistance;}
	void SetRadius(float fRadius) {Radius = fRadius;}
	void SetResolution(int nSlices, int nStacks) {Slices=nSlices; Stacks=nStacks;}
	void SetSun(bool bSun) {isSun=bSun;}
	void SetSunLight(GLenum eLight) { sunLight=eLight;}
	void SetPlaneTilt(float fTiltAng, float* pAxis) 
	{
		tiltPlanet = fTiltAng;
		planeTiltAxis[0] = pAxis[0];
		planeTiltAxis[1] = pAxis[1];
		planeTiltAxis[2] = pAxis[2];
	}
	void SetCurRotation(float curRot) {currentRotation = curRot;}
	void SetCurOrbit(float curOrbit) {currentOrbit = curOrbit;}
	string GetPlanetName (string pName) {return planetName;}
	int GetSlices() const { return Slices;}
	int GetStacks() const { return Stacks;}
	float GetRotationalSpeed() const {return rotationalSpeed;}
	float GetOrbitSpeed() const {return orbitSpeed;}
	float GetOrbitDistance() const {return orbitDistance;}
	float GetRadius() const {return Radius;}
	bool IsSun() const { return isSun;}
	GLenum GetSunLight() const {return sunLight;}
	void GetPlaneTilt(float& fTiltAng, float* pAxis) const 
	{
		fTiltAng = tiltPlanet;
		pAxis[0] = planeTiltAxis[0];
		pAxis[1] = planeTiltAxis[1];
		pAxis[2] = planeTiltAxis[2];
	}
	float GetCurRotation() const {return currentRotation;}
	float GetCurOrbit() const {return currentOrbit;}
	void Update();
	CCelestialBody();
	virtual ~CCelestialBody();

private:
	string planetName;
	int GetTime();
	void DrawBody();
	float rotationalSpeed;
	float orbitSpeed;
	float orbitDistance;
	float Radius;
	float currentRotation;
	float currentOrbit;
	float tiltPlanet;
	float planeTiltAxis[3];
	int lastUpdate;
	float pAmbient[4];
	float pDiffuse[4];
	float pSpecular[4];
	float pEmission[4];
	float Shininess;
	int Slices;
	int Stacks;
	bool isSun;
	GLenum sunLight;
	unsigned int m_nTexture;
	CCelestialBodyList listChildren;
	CCelestialBody* pParent;
};

const double PI = 3.1415926535898;


#endif // !defined(AFX_PLANET_H__820496D2_8A07_45EA_8E0B_2416C1485B26__INCLUDED_)
